/*
 * trAIns - An AI for OpenTTD
 * Copyright (C) 2009  Luis Henrique O. Rios
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/


class Land {
	t = null;
	swaped_w_h = null;

	constructor(){
		t = array(4);
	}

	function _cmp(other){
		if(this.t[0] == other.t[0]){
			if(this.swaped_w_h == other.swaped_w_h) return 0;
			else if(this.swaped_w_h) return 1;
			else return -1;
		}else if(this.t[0] > other.t[0]) return 1;
		else return -1;
	}

	function _tostring(){
		return Tile.ToString(t[0]) + " " + Tile.ToString(t[1]) + " " + Tile.ToString(t[2]) +
			" " + Tile.ToString(t[3]);
	}
}

class LandAllocation {
	cost = null;
	benefit = null;
	land = null;
	diagonal = null;

	static function compare(lt1 , lt2){
		if(lt1.cost == lt2.cost) return 0;
		else if(lt1.cost >= lt2.cost) return 1;
		else return -1;
	}

	function _cmp(other){
		return compare(this , other);
	}
}

class LandAllocator {
	/* Public: */

	constructor(w , h , land_validator , land_validator_param , max_cost , can_swap_w_h , cost_priority){
		this.w = w;
		this.h = h;
		this.land_validator = land_validator;
		this.land_validator_param = land_validator_param;
		this.max_cost = max_cost;
		this.can_swap_w_h = can_swap_w_h && w != h;
		this.cost_priority = cost_priority;
		checked_lands = BinaryTree();
		land_allocations = array(0);
		GenerateLandOffsets();
	}

	/* Private: */

	checked_lands = null;
	land_allocations = null;
	land_offsets = null;
	w = null;
	h = null;
	land_validator = null;
	land_validator_param = null;
	max_cost = null;
	can_swap_w_h = null;
	cost_priority = null;

	static land_allocations_max_size = 200;
	static diagonals = [[0 , 3] , [1 , 2] , [3 , 0] , [2 , 1]];

	function Evaluation(c , b);
	function GenerateLandOffsets();
	function GenerateTilesLands(tile);
	function GetBestLand();
	static function MyLevelLand(start_tile , end_tile);
	static function IsBuildableArea(initial_tile , area_w , area_h);
}

class IndustryLandAllocator {
	/* Public: */
	constructor(w , h , land_validator , land_validator_param , terraforming_max_cost , can_swap_w_h ,
		cost_priority , coverage_radius , industry){

		land_allocator = LandAllocator(w , h , land_validator , land_validator_param ,
			terraforming_max_cost , can_swap_w_h , cost_priority);

		this.industry = industry;
		this.coverage_radius = coverage_radius;
	}

	function AllocateLand();

	/* Private: */
	land_allocator = null;
	industry = null;
	coverage_radius = null;
}

function IndustryLandAllocator::AllocateLand(){
	local initial_tile = AIIndustry.GetLocation(industry) +
		AIMap.GetTileIndex(-coverage_radius , -coverage_radius);
	local area_w = 4 + coverage_radius * 2;
	local area_h = 4 + coverage_radius * 2;
	local exit = false;

 	for(local w = 0 ; w < area_w && !exit ; w++){
		for(local h = 0 ; h < area_h && !exit ; h++){
			local tile = initial_tile + AIMap.GetTileIndex(w , h);
			exit = land_allocator.GenerateTilesLands(tile);
		}
	}

	return land_allocator.GetBestLand();
}

function LandAllocator::IsBuildableArea(initial_tile , area_w , area_h){
 	for(local w = 0 ; w < area_w ; w++){
		for(local h = 0 ; h < area_h ; h++){
			local tile = initial_tile + AIMap.GetTileIndex(w , h);
			if(!AIMap.IsValidTile(tile) || !AITile.IsBuildable(tile)) return false;
		}
	}
	return true;
}

function LandAllocator::MyLevelLand(start_tile , end_tile){
	if(AIMap.GetTileX(start_tile) <= AIMap.GetTileX(end_tile)){
		end_tile += AIMap.GetTileIndex(1 , 0);
	}else{
		start_tile += AIMap.GetTileIndex(1 , 0);
	}

	if(AIMap.GetTileY(start_tile) <= AIMap.GetTileY(end_tile)){
		end_tile += AIMap.GetTileIndex(0 , 1);
	}else{
		start_tile += AIMap.GetTileIndex(0 , 1);
	}

	if(AITile.GetCornerHeight(start_tile , AITile.CORNER_N) == 0){
		return false;
	}

	/* Check if is already flat. */
	{
		local ex = AIMap.GetTileX(end_tile);
		local ey = AIMap.GetTileY(end_tile);
		local sx = AIMap.GetTileX(start_tile);
		local sy = AIMap.GetTileY(start_tile);
		local initial_tile , area_w , area_h;
		local exit = false;
		local start_tile_height = AITile.GetCornerHeight(start_tile , AITile.CORNER_N);

		if(sx > ex){
			local aux = sx;
			sx = ex;
			ex = aux;
		}

		if(sy > ey){
			local aux = sy;
			sy = ey;
			ey = aux;
		}


		initial_tile = AIMap.GetTileIndex(sx , sy);
		area_w = ex - sx + 1;
		area_h = ey - sy + 1;

		for(local w = 0 ; w < area_w && !exit ; w++){
			for(local h = 0 ; h < area_h && !exit ; h++){
				local tile = initial_tile + AIMap.GetTileIndex(w , h);
				if(start_tile_height != AITile.GetCornerHeight(tile , AITile.CORNER_N)) exit = true;
			}
		}

		if(!exit) return true;
	}

	return AITile.LevelTiles(start_tile , end_tile);
}

function LandAllocator::GenerateLandOffsets(){
	local w_less_1 = w - 1;
	local h_less_1 = h - 1;

	if(can_swap_w_h){
		land_offsets = array(8);
	}else{
		land_offsets = array(4);
	}

	land_offsets[0] = array(4);
	land_offsets[0][0] = AIMap.GetTileIndex(-w_less_1 , -h_less_1);
	land_offsets[0][1] = AIMap.GetTileIndex(0 , -h_less_1);
	land_offsets[0][2] = AIMap.GetTileIndex(-w_less_1 , 0);
	land_offsets[0][3] = AIMap.GetTileIndex(0 , 0);

	land_offsets[1] = array(4);
	land_offsets[1][0] = AIMap.GetTileIndex(0 , -h_less_1);
	land_offsets[1][1] = AIMap.GetTileIndex(w_less_1 , -h_less_1);
	land_offsets[1][2] = AIMap.GetTileIndex(0 , 0);
	land_offsets[1][3] = AIMap.GetTileIndex(w_less_1 , 0);

	land_offsets[2] = array(4);
	land_offsets[2][0] = AIMap.GetTileIndex(0 , 0);
	land_offsets[2][1] = AIMap.GetTileIndex(w_less_1 , 0);
	land_offsets[2][2] = AIMap.GetTileIndex(0 , h_less_1);
	land_offsets[2][3] = AIMap.GetTileIndex(w_less_1 , h_less_1);

	land_offsets[3] = array(4);
	land_offsets[3][0] = AIMap.GetTileIndex(-w_less_1 , 0);
	land_offsets[3][1] = AIMap.GetTileIndex(0 , 0);
	land_offsets[3][2] = AIMap.GetTileIndex(-w_less_1 , h_less_1);
	land_offsets[3][3] = AIMap.GetTileIndex(0 , h_less_1);

	if(!can_swap_w_h) return;

	land_offsets[4] = array(4);
	land_offsets[4][0] = AIMap.GetTileIndex(-h_less_1 , -w_less_1);
	land_offsets[4][1] = AIMap.GetTileIndex(0 , -w_less_1);
	land_offsets[4][2] = AIMap.GetTileIndex(-h_less_1 , 0);
	land_offsets[4][3] = AIMap.GetTileIndex(0 , 0);

	land_offsets[5] = array(4);
	land_offsets[5][0] = AIMap.GetTileIndex(0 , -w_less_1);
	land_offsets[5][1] = AIMap.GetTileIndex(h_less_1 , -w_less_1);
	land_offsets[5][2] = AIMap.GetTileIndex(0 , 0);
	land_offsets[5][3] = AIMap.GetTileIndex(h_less_1 , 0);

	land_offsets[6] = array(4);
	land_offsets[6][0] = AIMap.GetTileIndex(0 , 0);
	land_offsets[6][1] = AIMap.GetTileIndex(h_less_1 , 0);
	land_offsets[6][2] = AIMap.GetTileIndex(0 , w_less_1);
	land_offsets[6][3] = AIMap.GetTileIndex(h_less_1 , w_less_1);

	land_offsets[7] = array(4);
	land_offsets[7][0] = AIMap.GetTileIndex(-h_less_1 , 0);
	land_offsets[7][1] = AIMap.GetTileIndex(0 , 0);
	land_offsets[7][2] = AIMap.GetTileIndex(-h_less_1 , w_less_1);
	land_offsets[7][3] = AIMap.GetTileIndex(0 , w_less_1);
}

function LandAllocator::GenerateTilesLands(tile){
	local ai_test_mode = AITestMode();

	for(local i = 0 ; i < land_offsets.len() ; i++){
		local land = Land();
		local buildable_area;
		local w , h;

		land.t[0] = tile + land_offsets[i][0];
		land.t[1] = tile + land_offsets[i][1];
		land.t[2] = tile + land_offsets[i][2];
		land.t[3] = tile + land_offsets[i][3];
		land.swaped_w_h = (i >= 4);

		if(!AIMap.IsValidTile(land.t[0]) || checked_lands.Exists(land)) continue;

		if(land.swaped_w_h){
			w = this.h;
			h = this.w;
		}else{
			w = this.w;
			h = this.h;
		}
		if(IsBuildableArea(land.t[0] , w , h)){
			local user_evaluation;
			user_evaluation = land_validator(land , land_validator_param);
			if(user_evaluation == 0.0) continue;

			for(local j = 0 ; j < 4 ; j++){
				local cost = AIAccounting();

				if(MyLevelLand(land.t[diagonals[j][0]] , land.t[diagonals[j][1]]) &&
					cost.GetCosts() <= max_cost){

					local land_allocation = LandAllocation();

					land_allocation.cost = cost.GetCosts().tofloat();
					land_allocation.benefit = user_evaluation;
					land_allocation.land = land;
					land_allocation.diagonal = j;

					land_allocations.push(land_allocation);

					if(land_allocations.len() >= land_allocations_max_size) return true;
				}
			}
		}

		checked_lands.Insert(land);
	}
	return false;
}

function LandAllocator::Evaluation(c , b){
	return cost_priority * c + (1.0 - cost_priority) * b;
}

function LandAllocator::GetBestLand(){
	local biggest_cost = -1.0;

	if(land_allocations.len() <= 0) return null;
	foreach(land_allocation in land_allocations){
		if(land_allocation.cost > biggest_cost) biggest_cost = land_allocation.cost;
	}

	if(biggest_cost == 0.0) biggest_cost = 1.0;

	foreach(land_allocation in land_allocations){
		land_allocation.cost =
			Evaluation(1.0 - (land_allocation.cost/biggest_cost) , land_allocation.benefit);
	}

	land_allocations.sort(LandAllocation.compare);
	local land_allocation = land_allocations.pop();
	local diagonal = land_allocation.diagonal;

	if(MyLevelLand(land_allocation.land.t[diagonals[diagonal][0]] ,
		land_allocation.land.t[diagonals[diagonal][1]])){
		return land_allocation.land;
	}

	return null;
}
