/*
 * trAIns - An AI for OpenTTD
 * Copyright (C) 2009  Luis Henrique O. Rios
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/


require("log_messages_manager.nut");
require("error_manager.nut");
require("industry_manager.nut");
require("money_manager.nut");
require("land_allocator.nut");
require("scheduler.nut");
require("vehicle_seller.nut");

require("railroad/aystar.nut");
require("railroad/double_railroad_builder.nut");
require("railroad/double_railroad_depot_builder.nut");
require("railroad/double_railroad_junction_builder.nut");
require("railroad/double_railroad_station_builder.nut");
require("railroad/industry_usage.nut");
require("railroad/railroad_common.nut");
require("railroad/railroad_double_track_parts.nut");
require("railroad/railroad_manager.nut");

require("util/binary_heap.nut");
require("util/binary_tree.nut");
require("util/direction.nut");
require("util/hash_table.nut");
require("util/math.nut");
require("util/pair.nut");
require("util/tile.nut");

/* Global: */
ai_instance <- null;

class trAIns extends AIController {
	/* Public: */
	dtp = null;
	money_manager = null;
	industry_manager = null;
	scheduler = null;
	vehicle_seller = null;

	function Start();
	function Looping();
	constructor(){
		ai_instance = this;

		dtp = DoubleTrackParts();
		money_manager = MoneyManager();
		industry_manager = IndustryManager();
		scheduler = Scheduler();
		railroad_manager = RailroadManager();
		vehicle_seller = VehicleSeller();

		scheduler.CreateTask(Looping , null , Scheduler.ANUAL_INTERVAL);
		scheduler.CreateTask(MoneyManager.Loop , money_manager , Scheduler.BIANNUAL_INTERVAL);
		scheduler.CreateTask(VehicleSeller.Loop , vehicle_seller , Scheduler.MONTHLY_INTERVAL);
	}

	/* Private: */
	railroad_manager = null;
}

function trAIns::Looping(dummy){
	LogMessagesManager.PrintLogMessage("Looping...");
	return false;
}

function trAIns::Start() {
	/* Check some options. */
	if(AIGameSettings.IsDisabledVehicleType(AIVehicle.VT_RAIL) ||
		(AIGameSettings.IsValid("vehicle.max_trains") &&
		AIGameSettings.GetValue("vehicle.max_trains") == 0)){

		LogMessagesManager.PrintErrorMessage("I can only play with trains and they are disabled.");
		return;
	}

	while(true){
		/* Handle the events. */
		while(AIEventController.IsEventWaiting()) {
			local e = AIEventController.GetNextEvent();
			switch(e.GetEventType()){
				case AIEvent.AI_ET_VEHICLE_WAITING_IN_DEPOT:
					local vehicle_id = AIEventVehicleWaitingInDepot.Convert(e).GetVehicleID();
					vehicle_seller.TreatEventVehicleWaitingInDepot(vehicle_id);
				break;
				case AIEvent.AI_ET_INDUSTRY_OPEN:
					local industry_id = AIEventIndustryOpen.Convert(e).GetIndustryID();
					industry_manager.IndustryOpen(industry_id);
				break;
				case AIEvent.AI_ET_INDUSTRY_CLOSE:
					local industry_id = AIEventIndustryClose.Convert(e).GetIndustryID();
					industry_manager.IndustryClose(industry_id);
					railroad_manager.InformIndustryClosure(industry_id);
				break;
				case AIEvent.AI_ET_ENGINE_PREVIEW:
					AIEventEnginePreview.Convert(e).AcceptPreview();
				break;
			}
		}

		scheduler.Loop();
	}
}
