/*
 * trAIns - An AI for OpenTTD
 * Copyright (C) 2009  Luis Henrique O. Rios
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/


class MoneyManager {
	/* Public: */
	constructor(){
		first_reservation_id = total_money_reserved = 0;
		reservations = AIList();
	}

	function ReleaseReservation(reservation_id);
	function ReserveMoney(amount_to_reserve = 0);
	function Loop(self);

	/* Private: */
	total_money_reserved = null;
	reservations = null;
	first_reservation_id = null;
}

function MoneyManager::ReserveMoney(amount_to_reserve = 0){
	if(amount_to_reserve == 0){
		amount_to_reserve = AICompany.GetBankBalance(AICompany.COMPANY_SELF) + AICompany.GetMaxLoanAmount() -
			AICompany.GetLoanAmount() - total_money_reserved;
		if(amount_to_reserve <= 0) return null;
		AICompany.SetMinimumLoanAmount(AICompany.GetMaxLoanAmount());
	}else{
		if(AICompany.GetBankBalance(AICompany.COMPANY_SELF) + AICompany.GetMaxLoanAmount() -
			AICompany.GetLoanAmount() - total_money_reserved < amount_to_reserve) return null;
		if(AICompany.GetBankBalance(AICompany.COMPANY_SELF) - total_money_reserved < amount_to_reserve){
			local amount_to_loan;
			amount_to_loan = total_money_reserved + amount_to_reserve -
				AICompany.GetBankBalance(AICompany.COMPANY_SELF)
			AICompany.SetMinimumLoanAmount(AICompany.GetLoanAmount() + amount_to_loan);
		}
	}
	reservations.AddItem(first_reservation_id , amount_to_reserve);
	total_money_reserved += amount_to_reserve;
	return first_reservation_id++;
}

function MoneyManager::ReleaseReservation(reservation_id){
	local amount_reserved;
	assert(reservation_id != null && reservations.HasItem(reservation_id));
	amount_reserved = reservations.GetValue(reservation_id);
	reservations.RemoveItem(reservation_id);
	total_money_reserved -= amount_reserved;
}

function MoneyManager::Loop(self){
	this = self;
	if(reservations.Count() == 0 && AICompany.GetLoanAmount() > 0){
		local amount_to_pay = min(AICompany.GetBankBalance(AICompany.COMPANY_SELF) , AICompany.GetLoanAmount());
		amount_to_pay -= (amount_to_pay % AICompany.GetLoanInterval());
		if(amount_to_pay != 0){
			amount_to_pay = AICompany.GetLoanAmount() - amount_to_pay;
			AICompany.SetLoanAmount(amount_to_pay);
		}
	}
	return false;
}
